﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.ParameterConfigurationEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MetaOptimization {
  /// <summary>
  /// TODO
  /// </summary>
  [Item("AlgorithmSubScopesCreator", "TODO")]
  [StorableClass]
  public class AlgorithmSubScopesCreator : SingleSuccessorOperator {
    #region Parameter properties
    public ILookupParameter<TypeValue> AlgorithmTypeParameter {
      get { return (ILookupParameter<TypeValue>)Parameters[MetaOptimizationProblem.AlgorithmTypeParameterName]; }
    }
    public ILookupParameter<IItemList<IProblem>> ProblemsParameter {
      get { return (ILookupParameter<IItemList<IProblem>>)Parameters[MetaOptimizationProblem.ProblemsParameterName]; }
    }
    public ILookupParameter<ParameterConfigurationTree> ParameterConfigurationParameter {
      get { return (ILookupParameter<ParameterConfigurationTree>)Parameters["ParameterConfigurationTree"]; }
    }
    public LookupParameter<IntValue> RepetitionsParameter {
      get { return (LookupParameter<IntValue>)Parameters[MetaOptimizationProblem.RepetitionsParameterName]; }
    }
    private ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    #endregion

    #region Constructors and Cloning
    public AlgorithmSubScopesCreator()
      : base() {
      Parameters.Add(new LookupParameter<TypeValue>(MetaOptimizationProblem.AlgorithmTypeParameterName, ""));
      Parameters.Add(new LookupParameter<IItemList<IProblem>>(MetaOptimizationProblem.ProblemsParameterName, ""));
      Parameters.Add(new LookupParameter<ParameterConfigurationTree>("ParameterConfigurationTree", ""));
      Parameters.Add(new LookupParameter<IntValue>(MetaOptimizationProblem.RepetitionsParameterName, "Number of evaluations on one problem."));
      Parameters.Add(new ScopeParameter("CurrentScope", "The current scope whose sub-scopes represent the parents."));
    }

    [StorableConstructor]
    protected AlgorithmSubScopesCreator(bool deserializing) : base(deserializing) { }
    protected AlgorithmSubScopesCreator(AlgorithmSubScopesCreator original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AlgorithmSubScopesCreator(this, cloner);
    }
    #endregion

    public override IOperation Apply() {
      ParameterConfigurationTree parameterConfiguration = ParameterConfigurationParameter.ActualValue;
      IItemList<IProblem> problems = ProblemsParameter.ActualValue;
      int repetitions = RepetitionsParameter.ActualValue.Value;
      Type algorithmType = AlgorithmTypeParameter.ActualValue.Value;

      for (int i = 0; i < repetitions; i++) {
        for (int j = 0; j < problems.Count; j++) {
          IScope child = new Scope(string.Format("Problem {0}, Repetition {1}", j, i));
          var algorithm = MetaOptimizationUtil.CreateParameterizedAlgorithmInstance(parameterConfiguration, algorithmType, (IProblem)problems[j].Clone());
          child.Variables.Add(new Variable("Algorithm", algorithm));
          child.Variables.Add(new Variable("ProblemIndex", new IntValue(j)));
          child.Variables.Add(new Variable("RepetitionIndex", new IntValue(i)));
          CurrentScope.SubScopes.Add(child);
        }
      }
      return base.Apply();
    }
  }
}